/*
 * Copyright (c) 2005-2013, BearWare.dk
 * 
 * Contact Information:
 *
 * Bjoern D. Rasmussen
 * Bedelundvej 79
 * DK-9830 Taars
 * Denmark
 * Email: contact@bearware.dk
 * Phone: +45 20 20 54 59
 * Web: http://www.bearware.dk
 *
 * This source code is part of the TeamTalk 4 SDK owned by
 * BearWare.dk. All copyright statements may not be removed 
 * or altered from any source distribution. If you use this
 * software in a product, an acknowledgment in the product 
 * documentation is required.
 *
 */

#include "uservolumedlg.h"
#include "mainwindow.h"

#include <QMessageBox>

extern TTInstance* ttInst;

UserVolumeDlg::UserVolumeDlg(int userid, QWidget * parent/* = 0*/)
: QDialog(parent, Qt::Dialog | Qt::WindowTitleHint)
, m_userid(userid)
{
    ui.setupUi(this);
    setWindowIcon(QIcon(APPICON));

    connect(ui.volumeSlider, SIGNAL(valueChanged(int)),
            SLOT(slotVolumeChanged(int)));
    connect(ui.leftChkBox, SIGNAL(clicked()), SLOT(slotMuteChannel()));
    connect(ui.rightChkBox, SIGNAL(clicked()), SLOT(slotMuteChannel()));

    User user;
    if(TT_GetUser(ttInst, m_userid, &user))
        setWindowTitle(windowTitle() + QString(" - ") + _Q(user.szNickname));
    ui.volumeSlider->setValue(TT_GetUserVolume(ttInst, m_userid));

    BOOL left = false, right = false;
    TT_GetUserStereo(ttInst, userid, &left, &right);
    ui.leftChkBox->setChecked(!left);
    ui.rightChkBox->setChecked(!right);

    //don't show stereo option in mono-channel
    Channel chan;
    if(!TT_GetChannel(ttInst, user.nChannelID, &chan))
        return;

    bool stereo = false;
    switch(chan.codec.nCodec)
    {
    case SPEEX_CODEC :
        stereo = chan.codec.speex.bStereoPlayback; break;
    case CELT_CODEC :
        stereo = chan.codec.celt.nChannels>1; break;
    default :
        stereo = false;
    }
    ui.leftChkBox->setEnabled(stereo);
    ui.rightChkBox->setEnabled(stereo);  
}

void UserVolumeDlg::slotVolumeChanged(int vol)
{
    if(!TT_SetUserVolume(ttInst, m_userid, vol))
        QMessageBox::critical(this, tr("Volume"), 
        tr("Failed to change user's volume"));
}

void UserVolumeDlg::slotMuteChannel()
{
    TT_SetUserStereo(ttInst, m_userid, !ui.leftChkBox->isChecked(), 
                     !ui.rightChkBox->isChecked());
}
