using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using BearWare;

namespace TeamTalk.NET
{
    struct ItemData
    {
        public string text;
        public int id;
        public ItemData(string text, int id)
        {
            this.text = text;
            this.id = id;
        }

        public override string ToString() 
        {
            return text;
        }
    };

    public partial class PreferencesDlg : Form
    {
        TeamTalk4 ttclient;
        Settings settings;
        VideoCaptureDevice[] videodevs;

        public PreferencesDlg(TeamTalk4 tt, Settings settings)
        {
            ttclient = tt;
            this.settings = settings;
            InitializeComponent();
            this.CenterToScreen();

            dsoundRadioButton.Checked = settings.soundsystem == SoundSystem.SOUNDSYSTEM_DSOUND;
            winmmRadioButton.Checked = settings.soundsystem == SoundSystem.SOUNDSYSTEM_WINMM;

            UpdateSoundSystem(null, null);

            vidcodecComboBox.SelectedIndex = 0;
            this.vidqualityTrackBar.Value = settings.codec.theora.nQuality;
            this.vidbitrateNumericUpDown.Value = settings.codec.theora.nTargetBitRate/1024;
            UpdateVideoCaptureDevices();

            p2paudioCheckBox.Checked = (ttclient.GetFlags() & ClientFlag.CLIENT_P2P_AUDIO) == ClientFlag.CLIENT_P2P_AUDIO;
            p2pvideoCheckBox.Checked = (ttclient.GetFlags() & ClientFlag.CLIENT_P2P_VIDEO) == ClientFlag.CLIENT_P2P_VIDEO;
            int tcp = 0, udp = 0;
            if (ttclient.GetKeepAliveInterval(out tcp, out udp))
            {
                tcpkeepaliveNumericUpDown.Value = tcp;
                udpkeepaliveNumericUpDown.Value = udp;
            }
        }

        void UpdateSoundSystem(object sender, EventArgs e)
        {
            sndinputComboBox.Items.Clear();
            sndoutputComboBox.Items.Clear();

            SoundDevice[] input, output;
            ttclient.GetSoundInputDevices(out input);
            ttclient.GetSoundOutputDevices(out output);

            SoundSystem soundsystem = SoundSystem.SOUNDSYSTEM_DSOUND;
            if (winmmRadioButton.Checked)
            {
                soundsystem = SoundSystem.SOUNDSYSTEM_WINMM;
                Debug.WriteLine("WinMM devices");
            }
            else
                Debug.WriteLine("DirectSound devices");
            
            Debug.WriteLine("INPUT DEVICES");
            foreach (SoundDevice dev in input)
            {
                if (dev.nSoundSystem != soundsystem)
                    continue;
                Debug.WriteLine("Name " + dev.szDeviceName);
                Debug.WriteLine("\tID #" + dev.nDeviceID);
                Debug.WriteLine("\tUnique ID #" + dev.szDeviceID);
                Debug.WriteLine("\tWaveDeviceID #" + dev.nWaveDeviceID);
                string tmp;
                if (WindowsMixer.GetWaveInName(dev.nWaveDeviceID, out tmp))
                    Debug.WriteLine("\tMixer name: " + tmp);
                for (int i = 0; i < WindowsMixer.GetWaveInControlCount(dev.nWaveDeviceID); i++)
                    if (WindowsMixer.GetWaveInControlName(dev.nWaveDeviceID, i, out tmp))
                    {
                        Debug.WriteLine("\t\tControl name: " + tmp);
                        Debug.WriteLine("\t\tSelected: " + WindowsMixer.GetWaveInControlSelected(dev.nWaveDeviceID, i));
                    }

                int index = sndinputComboBox.Items.Add(new ItemData(dev.szDeviceName, dev.nDeviceID));
                if (dev.nDeviceID == settings.sndinputid)
                    sndinputComboBox.SelectedIndex = index;
            }

            Debug.WriteLine("OUTPUT DEVICES");
            foreach (SoundDevice dev in output)
            {
                if (dev.nSoundSystem != soundsystem)
                    continue;
                Debug.WriteLine("Name " + dev.szDeviceName);
                Debug.WriteLine("\tID #" + dev.nDeviceID);
                Debug.WriteLine("\tUnique ID #" + dev.szDeviceID);
                Debug.WriteLine("\tWaveDeviceID #" + dev.nWaveDeviceID);
                string tmp;
                if(WindowsMixer.GetWaveOutName(dev.nWaveDeviceID, out tmp))
                    Debug.WriteLine("\tMixer name: " + tmp);
                int index = sndoutputComboBox.Items.Add(new ItemData(dev.szDeviceName, dev.nDeviceID));
                if (dev.nDeviceID == settings.sndoutputid)
                    sndoutputComboBox.SelectedIndex = index;
            }
            if (sndinputComboBox.SelectedIndex < 0 && sndinputComboBox.Items.Count>0)
                sndinputComboBox.SelectedIndex = 0;
            if (sndoutputComboBox.SelectedIndex < 0 && sndoutputComboBox.Items.Count > 0)
                sndoutputComboBox.SelectedIndex = 0;
        }

        private void sndTestCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            ItemData input = (ItemData) sndinputComboBox.SelectedItem;
            ItemData output = (ItemData)sndoutputComboBox.SelectedItem;
            if (sndTestCheckBox.Checked)
            {
                if (duplexCheckBox.Checked)
                {
                    AudioConfig audcfg;
                    audcfg.bEnableAGC = true;
                    audcfg.nGainLevel = TeamTalk4.DEFAULT_AGC_GAINLEVEL;
                    audcfg.nMaxIncDBSec = TeamTalk4.DEFAULT_AGC_INC_MAXDB;
                    audcfg.nMaxDecDBSec = TeamTalk4.DEFAULT_AGC_DEC_MAXDB;
                    audcfg.nMaxGainDB = TeamTalk4.DEFAULT_AGC_GAINMAXDB;
                    audcfg.bEnableDenoise = true;
                    audcfg.nMaxNoiseSuppressDB = TeamTalk4.DEFAULT_DENOISE_SUPPRESS;
                    if (!ttclient.StartSoundLoopbackTestEx(input.id, output.id, 16000, 1,
                        audcfg, echocancelCheckBox.Checked))
                    {
                        MessageBox.Show("Failed to test selected device");
                        sndTestCheckBox.Checked = false;
                    }
                }
                else
                {
                    if (!ttclient.StartSoundLoopbackTest(input.id, output.id, 16000, 1))
                    {
                        MessageBox.Show("Failed to test selected device");
                        sndTestCheckBox.Checked = false;
                    }
                }
            }
            else
                ttclient.StopSoundLoopbackTest();
        }

        private void PreferencesDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            ttclient.StopSoundLoopbackTest();
        }

        private void UpdateVideoCaptureDevices()
        {
            if (!ttclient.GetVideoCaptureDevices(out videodevs) || videodevs.Length == 0)
                return;

            foreach (VideoCaptureDevice dev in videodevs)
            {
                int index = viddevComboBox.Items.Add(dev.szDeviceName);
                if (dev.szDeviceID == settings.videoid)
                    viddevComboBox.SelectedIndex = index;
            }
            if(viddevComboBox.SelectedIndex<0)
                viddevComboBox.SelectedIndex = 0;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ClientFlag flags = ttclient.GetFlags();

            if ((ttclient.Flags & ClientFlag.CLIENT_SNDINOUTPUT_DUPLEX) ==
                ClientFlag.CLIENT_SNDINOUTPUT_DUPLEX)
                ttclient.CloseSoundDuplexDevices();
            else
            {
                ttclient.CloseSoundInputDevice();
                ttclient.CloseSoundOutputDevice();
            }

            ItemData inputItem = (ItemData)sndinputComboBox.SelectedItem;
            ItemData outputItem = (ItemData)sndoutputComboBox.SelectedItem;
            settings.sndinputid = inputItem.id;
            settings.sndoutputid = outputItem.id;

            if (duplexCheckBox.Checked)
            {
                if (!ttclient.InitSoundDuplexDevices(settings.sndinputid, settings.sndoutputid))
                    MessageBox.Show("Failed to init sound devices");

                ttclient.EnableEchoCancellation(echocancelCheckBox.Checked);
            }
            else
            {
                if (!ttclient.InitSoundInputDevice(settings.sndinputid))
                    MessageBox.Show("Failed to init sound input device");

                if (!ttclient.InitSoundOutputDevice(settings.sndoutputid))
                    MessageBox.Show("Failed to init sound output device");
            }
            settings.soundsystem = dsoundRadioButton.Checked ? SoundSystem.SOUNDSYSTEM_DSOUND : SoundSystem.SOUNDSYSTEM_WINMM;

            if (viddevComboBox.Items.Count > 0)
            {
                VideoCodec codec;
                codec.nCodec = Codec.THEORA_CODEC;
                codec.theora.nQuality = vidqualityTrackBar.Value;
                codec.theora.nTargetBitRate = (int)vidbitrateNumericUpDown.Value * 1024;

                CaptureFormat capformat = videodevs[viddevComboBox.SelectedIndex].captureFormats[formatComboBox.SelectedIndex];

                if ((ttclient.GetFlags() & ClientFlag.CLIENT_VIDEO_READY) == ClientFlag.CLIENT_VIDEO_READY &&
                   !(settings.videoid == videodevs[viddevComboBox.SelectedIndex].szDeviceID &&
                    Util.Equals(codec, settings.codec) &&
                    Util.Equals(capformat, settings.capformat)) )
                    ttclient.CloseVideoCaptureDevice();

                settings.codec.nCodec = Codec.THEORA_CODEC;
                settings.codec.theora.nQuality = vidqualityTrackBar.Value;
                settings.codec.theora.nTargetBitRate = (int)vidbitrateNumericUpDown.Value * 1024;

                settings.videoid = videodevs[viddevComboBox.SelectedIndex].szDeviceID;
                settings.capformat = capformat;
                if ((ttclient.GetFlags() & ClientFlag.CLIENT_VIDEO_READY) == 0)
                {
                    if(!ttclient.InitVideoCaptureDevice(settings.videoid, settings.capformat, settings.codec))
                    MessageBox.Show("Failed to initialize video capture device");
                }
            }

            ttclient.EnablePeerToPeer(TransmitType.TRANSMIT_AUDIO, p2paudioCheckBox.Checked);
            ttclient.EnablePeerToPeer(TransmitType.TRANSMIT_VIDEO, p2pvideoCheckBox.Checked);
            ttclient.SetKeepAliveInterval((int)tcpkeepaliveNumericUpDown.Value, (int)udpkeepaliveNumericUpDown.Value);
        }

        private void viddevComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            formatComboBox.Items.Clear();
            for (int i = 0; i < videodevs[viddevComboBox.SelectedIndex].nCaptureFormatsCount;i++ )
            {
                CaptureFormat cap = videodevs[viddevComboBox.SelectedIndex].captureFormats[i];
                int fps = cap.nFPS_Numerator / cap.nFPS_Denominator;
                string imgfmt = "";
                switch (cap.picFourCC)
                {
                    case FourCC.FOURCC_I420:
                        imgfmt = "I420"; break;
                    case FourCC.FOURCC_YUY2:
                        imgfmt = "YUY2"; break;
                    case FourCC.FOURCC_RGB32:
                        imgfmt = "RGB32"; break;
                }
                int index = formatComboBox.Items.Add(cap.nWidth + "x" + cap.nHeight + " FPS: " + fps + " " + imgfmt);
                if (Util.Equals(settings.capformat, cap))
                    formatComboBox.SelectedIndex = index;
            }
            if (formatComboBox.SelectedIndex < 0 && formatComboBox.Items.Count>0)
                formatComboBox.SelectedIndex = 0;
        }

        private void duplexCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            echocancelCheckBox.Enabled = duplexCheckBox.Checked;
        }
    }
}