using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BearWare;

namespace TeamTalk.NET
{
    public partial class UserAccountsDlg : Form
    {
        TeamTalk4 ttclient;
        int create_cmdid;
        int list_cmdid;
        int del_cmdid;

        UserAccount account_create;
        List<UserAccount> all_accounts = new List<UserAccount>();

        public UserAccountsDlg(TeamTalk4 tt)
        {
            ttclient = tt;
            create_cmdid = 0;
            del_cmdid = 0;

            InitializeComponent();
            this.CenterToScreen();

            listView1.Columns.Add("Username");
            listView1.Columns.Add("Password");
            listView1.Columns.Add("User Type");
            listView1.Columns.Add("UserData");
            listView1.Columns.Add("Note");

            int[] channelids;
            if(ttclient.GetServerChannels(out channelids))
            {
                string channel;
                foreach (int id in channelids)
                {
                    Channel chan; //auto-op only works for static channels
                    if (ttclient.GetChannel(id, out chan) &&
                        (chan.uChannelType & ChannelType.CHANNEL_STATIC) == ChannelType.CHANNEL_STATIC &&
                        ttclient.GetChannelPath(id, out channel))
                        availchanListBox.Items.Add(channel);
                }
            }

            ttclient.OnCmdProcessing += new TeamTalk4.CommandProcessing(ttclient_OnCmdProcessing);
            ttclient.OnCmdError += new TeamTalk4.CommandError(ttclient_OnCmdError);
            ttclient.OnCmdSuccess += new TeamTalk4.CommandSuccess(ttclient_OnCmdSuccess);

            //hopefully you have less than 100000 accounts
            list_cmdid = ttclient.DoListUserAccounts(0, 100000);
        }

        void ClearControls()
        {
            usernameTextBox.Text = "";
            passwordTextBox.Text = "";
            defuserRadioButton.Checked = true;
            userdataTextBox.Text = "0";
            noteTextBox.Text = "";
            initchanTextBox.Text = "";
            selchanListBox.Items.Clear();
        }

        void EnableControls(bool enable)
        {
            createButton.Enabled = enable;
            usernameTextBox.Enabled = enable;
            passwordTextBox.Enabled = enable;
            defuserRadioButton.Enabled = enable;
            adminRadioButton.Enabled = enable;
            noteTextBox.Enabled = enable;
            userdataTextBox.Enabled = enable;
            initchanTextBox.Enabled = enable;
            availchanListBox.Enabled = enable;
            selchanListBox.Enabled = enable;
        }

        void ttclient_OnCmdProcessing(int nCmdID, bool bComplete)
        {
            if (create_cmdid == nCmdID && bComplete)
            {
                EnableControls(true);
                ClearControls();
            }
            else if (list_cmdid == nCmdID && bComplete)
            {
                UserAccount[] accounts;
                if (ttclient.GetUserAccounts(out accounts))
                {
                    foreach (UserAccount account in accounts)
                        AddAccount(account);
                }
            }
            else if (del_cmdid == nCmdID && bComplete)
            {
                listView1.Enabled = true;
                if (listView1.SelectedItems.Count > 0)
                {
                    listView1.Items.Remove(listView1.SelectedItems[0]);
                }
            }
        }

        void ttclient_OnCmdSuccess(int nCmdID)
        {
            if (nCmdID == create_cmdid)
            {
                RemoveAccount(account_create.szUsername);
                AddAccount(account_create);
            }
            if (nCmdID == del_cmdid)
            {
                if(listView1.SelectedItems.Count>0)
                    RemoveAccount(listView1.SelectedItems[0].SubItems[0].Text);
            }
        }

        void ttclient_OnCmdError(ClientError nErrorNo, int nCmdID)
        {
            //clear create account if unsuccessful
            if (nCmdID == create_cmdid)
            {
                account_create = new UserAccount();
            }
        }

        private void createButton_Click(object sender, EventArgs e)
        {
            if(usernameTextBox.Text.Length == 0)
                return;

            account_create = new UserAccount(); ;
            account_create.szUsername = usernameTextBox.Text;
            account_create.szPassword = passwordTextBox.Text;
            if(adminRadioButton.Checked)
                account_create.uUserType = UserType.USERTYPE_ADMIN;
            else
                account_create.uUserType = UserType.USERTYPE_DEFAULT;
            account_create.szNote = noteTextBox.Text;
            account_create.nUserData = int.Parse(userdataTextBox.Text);
            account_create.szInitChannel = initchanTextBox.Text;
            //why is this nescessary?!?!?
            account_create.autoOperatorChannels = new int[TeamTalk4.TT_CHANNELS_OPERATOR_MAX];
            int c = 0;
            //process auto-operator channels
            foreach (string channel in selchanListBox.Items)
            {
                int id = ttclient.GetChannelIDFromPath(channel);
                if(id>0)
                    account_create.autoOperatorChannels[c++] = id;
            }

            create_cmdid = ttclient.DoNewUserAccount(account_create);
            if (create_cmdid > 0)
            {
                EnableControls(false);
            }
        }

        private void deleteAccountToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                del_cmdid = ttclient.DoDeleteUserAccount(listView1.SelectedItems[0].Text);
                listView1.Enabled = false;
            }
        }

        void AddAccount(UserAccount account)
        {
            ListViewItem item = new ListViewItem();
            item.Text = account.szUsername;
            item.SubItems.Add(account.szPassword);
            item.SubItems.Add((account.uUserType & UserType.USERTYPE_ADMIN) == UserType.USERTYPE_ADMIN ? "Admin" : "Default");
            item.SubItems.Add(account.nUserData.ToString());
            item.SubItems.Add(account.szNote);
            listView1.Items.Add(item);

            all_accounts.Add(account);
        }


        void RemoveAccount(string username)
        {
            for (int i = 0; i < all_accounts.Count; i++)
            {
                if (all_accounts[i].szUsername == username)
                {
                    all_accounts.RemoveAt(i);
                    break;
                }
            }
            foreach (ListViewItem item in listView1.Items)
            {
                if (item.SubItems[0].Text == username)
                    listView1.Items.Remove(item);
            }
        }

        private void addChanButton_Click(object sender, EventArgs e)
        {
            if(!selchanListBox.Items.Contains(availchanListBox.SelectedItem))
                selchanListBox.Items.Add(availchanListBox.SelectedItem);
        }

        private void delChanButton_Click(object sender, EventArgs e)
        {
            selchanListBox.Items.Remove(selchanListBox.SelectedItem);
        }

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(listView1.SelectedItems.Count == 0)
                return;
            ClearControls();

            string username = listView1.SelectedItems[0].SubItems[0].Text;
            foreach (UserAccount acc in all_accounts)
            {
                if (acc.szUsername == username)
                {
                    usernameTextBox.Text = acc.szUsername;
                    passwordTextBox.Text = acc.szPassword;
                    defuserRadioButton.Checked = (acc.uUserType & UserType.USERTYPE_DEFAULT) == UserType.USERTYPE_DEFAULT;
                    adminRadioButton.Checked = (acc.uUserType & UserType.USERTYPE_ADMIN) == UserType.USERTYPE_ADMIN;
                    userdataTextBox.Text = acc.nUserData.ToString();
                    noteTextBox.Text = acc.szNote;
                    initchanTextBox.Text = acc.szInitChannel;
                    foreach (int id in acc.autoOperatorChannels)
                    {
                        string channel;
                        if (ttclient.GetChannelPath(id, out channel))
                            selchanListBox.Items.Add(channel);
                    }
                    break;
                }
            }
        }

        private void clrButton_Click(object sender, EventArgs e)
        {
            ClearControls();
        }
    }
}